@@ -22,9 +22,8 @@ from presto.IO import readPrimerFile, printLog
 from presto.Multiprocessing import SeqResult, manageProcesses, feedSeqQueue, \
                                    processSeqQueue, collectSeqQueue
 
-
 def extractPrimers(data, start, length, rev_primer=False, mode='mask', barcode=False,
-                   barcode_field=default_barcode_field, primer_field=default_primer_field,
+                   barcode_length=None, barcode_field=default_barcode_field, primer_field=default_primer_field,
                    delimiter=default_delimiter):
     """
     Extracts primer sequences directly
@@ -36,6 +35,8 @@ def extractPrimers(data, start, length, rev_primer=False, mode='mask', barcode=F
       rev_primer : if True extract relative to the tail end of the sequence.
       mode : defines the action taken; one of 'cut', 'mask', 'tag' or 'trim'.
       barcode : if True add sequence preceding primer to description.
+      barcode_length : length of the barcode sequence. If None (default), the barcode
+                       sequence is the full sequence preceding the primer sequence.
       barcode_field : name of the output barcode annotation.
       primer_field : name of the output primer annotation.
       delimiter : a tuple of delimiters for (annotations, field/values, value lists).
@@ -43,6 +44,7 @@ def extractPrimers(data, start, length, rev_primer=False, mode='mask', barcode=F
     Returns:
       presto.Multiprocessing.SeqResult: result object.
     """
+
     # Define result object
     result = SeqResult(data.id, data.data)
 
@@ -53,7 +55,7 @@ def extractPrimers(data, start, length, rev_primer=False, mode='mask', barcode=F
         return result
 
     # Create output sequence
-    out_seq = maskSeq(align, mode=mode, barcode=barcode, barcode_field=barcode_field,
+    out_seq = maskSeq(align, mode=mode, barcode=barcode, barcode_length=barcode_length, barcode_field=barcode_field,
                       primer_field=primer_field, delimiter=delimiter)
     result.results = out_seq
     result.valid = True
@@ -79,7 +81,7 @@ def extractPrimers(data, start, length, rev_primer=False, mode='mask', barcode=F
 
 def alignPrimers(data, primers, primers_regex=None, max_error=default_primer_max_error,
                  max_len=default_primer_max_len, rev_primer=False, skip_rc=False, mode='mask',
-                 barcode=False, barcode_field=default_barcode_field, primer_field=default_primer_field,
+                 barcode=False, barcode_length=None, barcode_field=default_barcode_field, primer_field=default_primer_field,
                  gap_penalty=default_primer_gap_penalty, score_dict=getDNAScoreDict(mask_score=(0, 1), gap_score=(0, 0)),
                  delimiter=default_delimiter):
     """
@@ -95,6 +97,8 @@ def alignPrimers(data, primers, primers_regex=None, max_error=default_primer_max
       skip_rc : if True do not check reverse complement sequences.
       mode : defines the action taken; one of 'cut', 'mask', 'tag' or 'trim'.
       barcode : if True add sequence preceding primer to description.
+      barcode_length : length of the barcode sequence. If None (default), the barcode
+                       sequence is the full sequence preceding the primer sequence.
       barcode_field : name of the output barcode annotation.
       primer_field : name of the output primer annotation.
       gap_penalty : a tuple of positive (gap open, gap extend) penalties.
@@ -117,7 +121,7 @@ def alignPrimers(data, primers, primers_regex=None, max_error=default_primer_max
         return result
 
     # Create output sequence
-    out_seq = maskSeq(align, mode=mode, barcode=barcode, barcode_field=barcode_field,
+    out_seq = maskSeq(align, mode=mode, barcode=barcode, barcode_length=barcode_length, barcode_field=barcode_field,
                       primer_field=primer_field, delimiter=delimiter)
     result.results = out_seq
     result.valid = bool(align.error <= max_error) if len(out_seq) > 0 else False
@@ -146,7 +150,7 @@ def alignPrimers(data, primers, primers_regex=None, max_error=default_primer_max
 
 
 def scorePrimers(data, primers, max_error=default_primer_max_error, start=default_primer_start, rev_primer=False, mode='mask',
-                 barcode=False, barcode_field=default_barcode_field, primer_field=default_primer_field,
+                 barcode=False, barcode_length=None, barcode_field=default_barcode_field, primer_field=default_primer_field,
                  score_dict=getDNAScoreDict(mask_score=(0, 1), gap_score=(0, 0)),
                  delimiter=default_delimiter):
     """
@@ -160,6 +164,8 @@ def scorePrimers(data, primers, max_error=default_primer_max_error, start=defaul
       rev_primer : if True align with the tail end of the sequence.
       mode : defines the action taken; one of 'cut', 'mask', 'tag' or 'trim'.
       barcode : if True add sequence preceding primer to description.
+      barcode_length : length of the barcode sequence. If None (default), the barcode
+                       sequence is the full sequence preceding the primer sequence.
       barcode_field : name of the output barcode annotation.
       primer_field : name of the output primer annotation.
       score_dict : optional dictionary of {(char1, char2): score} alignment scores
@@ -180,7 +186,7 @@ def scorePrimers(data, primers, max_error=default_primer_max_error, start=defaul
         return result
 
     # Create output sequence
-    out_seq = maskSeq(align, mode=mode, barcode=barcode, barcode_field=barcode_field,
+    out_seq = maskSeq(align, mode=mode, barcode=barcode, barcode_length=barcode_length, barcode_field=barcode_field,
                       primer_field=primer_field, delimiter=delimiter)
     result.results = out_seq
     result.valid = bool(align.error <= max_error) if len(out_seq) > 0 else False
@@ -251,6 +257,8 @@ def maskPrimers(seq_file, primer_file, align_func, align_args={},
         log['BARCODE'] = align_args['barcode']
     if 'barcode' in align_args and align_args['barcode']:
         log['BARCODE_FIELD'] = align_args['barcode_field']
+    if 'barcode_length' in align_args and align_args['barcode_length']:
+        log['BARCODE_LENGTH'] = align_args['barcode_length']
     log['PRIMER_FIELD'] = align_args['primer_field']
     log['NPROC'] = nproc
     printLog(log)
@@ -375,6 +383,9 @@ def getArgParser():
     group_align.add_argument('--barcode', action='store_true', dest='barcode',
                               help='''Specify to annotate reads sequences with barcode sequences
                                    (unique molecular identifiers) found preceding the primer.''')
+    group_align.add_argument('--barcodelen', action='store', dest='barcode_length', type=int,
+                             help='''Length of the barcode sequence. If not specified, the
+                                   barcode sequence is the full sequence preceding the primer.''')
     group_align.add_argument('--bf', action='store', dest='barcode_field', default=default_barcode_field,
                              help='''Name of the barcode annotation field.''')
     group_align.add_argument('--pf', action='store', dest='primer_field', default=default_primer_field,
@@ -410,6 +421,9 @@ def getArgParser():
     group_score.add_argument('--barcode', action='store_true', dest='barcode',
                               help='''Specify to annotate reads sequences with barcode sequences
                                    (unique molecular identifiers) found preceding the primer.''')
+    group_score.add_argument('--barcodelen', action='store', dest='barcode_length', type=int,
+                             help='''Length of the barcode sequence. If not specified, the
+                                   barcode sequence is the full sequence preceding the primer.''')
     group_score.add_argument('--bf', action='store', dest='barcode_field', default=default_barcode_field,
                              help='''Name of the barcode annotation field.''')
     group_score.add_argument('--pf', action='store', dest='primer_field', default=default_primer_field,
@@ -440,6 +454,9 @@ def getArgParser():
     group_extract.add_argument('--barcode', action='store_true', dest='barcode',
                                help='''Specify to remove the sequence preceding the extracted region and
                                     annotate the read with that sequence.''')
+    group_extract.add_argument('--barcodelen', action='store', dest='barcode_length', type=int,
+                                help='''Length of the barcode sequence. If not specified, the
+                                        barcode sequence is the full sequence preceding the primer.''')
     group_extract.add_argument('--bf', action='store', dest='barcode_field', default=default_barcode_field,
                                help='''Name of the barcode annotation field.''')
     group_extract.add_argument('--pf', action='store', dest='primer_field', default=default_primer_field,
@@ -469,6 +486,7 @@ if __name__ == '__main__':
                                    'gap_penalty':args_dict['gap_penalty'],
                                    'mode': args_dict['mode'],
                                    'barcode': args_dict['barcode'],
+                                   'barcode_length': args_dict['barcode_length'],
                                    'barcode_field': args_dict['barcode_field'],
                                    'primer_field': args_dict['primer_field']}
 
@@ -479,6 +497,7 @@ if __name__ == '__main__':
         del args_dict['gap_penalty']
         del args_dict['mode']
         del args_dict['barcode']
+        del args_dict['barcode_length']
         del args_dict['barcode_field']
         del args_dict['primer_field']
     elif args_dict['align_func'] is scorePrimers:
@@ -488,6 +507,7 @@ if __name__ == '__main__':
                                    'mode': args_dict['mode'],
                                    'barcode': args_dict['barcode'],
                                    'barcode_field': args_dict['barcode_field'],
+                                   'barcode_length': args_dict['barcode_length'],
                                    'primer_field': args_dict['primer_field']}
         del args_dict['max_error']
         del args_dict['start']
@@ -495,6 +515,7 @@ if __name__ == '__main__':
         del args_dict['mode']
         del args_dict['barcode']
         del args_dict['barcode_field']
+        del args_dict['barcode_length']
         del args_dict['primer_field']
     elif args_dict['align_func'] is extractPrimers:
         args_dict['primer_file'] = None
@@ -504,12 +525,14 @@ if __name__ == '__main__':
                                    'mode':args_dict['mode'],
                                    'barcode': args_dict['barcode'],
                                    'barcode_field': args_dict['barcode_field'],
+                                   'barcode_length': args_dict['barcode_length'],
                                    'primer_field': args_dict['primer_field']}
         del args_dict['start']
         del args_dict['length']
         del args_dict['rev_primer']
         del args_dict['mode']
         del args_dict['barcode']
+        del args_dict['barcode_length']
         del args_dict['barcode_field']
         del args_dict['primer_field']
 
